
/* Simple program:  Create a blank window, wait for keypress, quit.

   Please see the SDL documentation for details on using the SDL API:
   /Developer/Documentation/SDL/docs.html
*/
   
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <string>
#include <iostream>
#include <vector>

#include "SDL.h"
#include "SDL_video.h"
#include "SDL_opengl.h"
#include "timer.h"

#include "musicback.h"


using namespace std;

int speaking;
const int SCREEN_WIDTH = 640;
const int SCREEN_HEIGHT = 480; 
const int SCREEN_BPP = 32; 

const int screenwidth=320;
const int screenheight=240;

SDL_Surface* screen = NULL; 

//The frames per second 
const int FRAMES_PER_SECOND = 60; 
int frame = 0;
Timer fps;


struct Pos {
    float x;
    float y;
	float a;
};

Pos player;
vector<Pos> emit;

float dist(Pos p1, Pos p2)
{
	return sqrt((p1.x-p2.x)*(p1.x-p2.x) + (p1.y-p2.y)*(p1.y-p2.y) );
}

float dot(Pos a, Pos b)
{
	return a.x*b.x+a.y*b.y;
}

float l(Pos p1)
{
	return sqrt(p1.x*p1.x+p1.y*p1.y);
}

const float LINE_LENGTH=15;	
const float PI=3.1415926535;

float ang(Pos p1, Pos p2)
{
	Pos v1, v2;
	float a = atan2f(p2.y-p1.y,p2.x-p1.x)-atan2f(sin(p1.a)*10.0f,cos(p1.a)*10.0f);
	if (a>PI)
		a-=2*PI;
	if (a<-PI)
		a+=2*PI;
	return a;
}

float angtopan(float a)
{
	if (a>PI/2)
		a=PI-a;
	if (a<-PI/2)
		a=-PI - a;
	return a*0.9f/(PI/2);
}

float ramp(float min, float max,float minat, float maxat,float f)
{
	if (f<=minat) return min;
	if (f>=maxat) return max;
	
	return min + (max-min)*(f-minat)/(maxat-minat);
}

void updateSounds()
{
	for (int i=0;i<emit.size();i++)
	{
		float d = dist(player,emit[i]);
		float v =  ramp(0.3,0,0,200,d);
		set_emitter(i, v, ramp(0,angtopan(ang(player,emit[i])),2,4,d),521.0f);
	
	}
}

enum KEYS {
    K_LEFT,
	K_RIGHT,
	K_UP,
	K_DOWN,
	K_SPACE,
	K_MAX
};

bool keydown[K_MAX];

void InitGame()
{	
	player.x=-50-4*100;
	player.y=-50-4*100;
	player.a=PI/2;
	
	for (int i=-4;i<=4;i++)
	{
		for (int j=-4;j<=4;j++)
		{
			Pos e;

			e.x=50+100*i;
			e.y=50+100*j;

			emit.push_back(e);
		}
	}
	
	
     for (int i=0;i<K_MAX;i++)
     {
        keydown[i]==false;
     }
}

void GameLogic()
{
        if (keydown[K_LEFT]&&keydown[K_RIGHT])
        {
        }
        else if (keydown[K_LEFT])
        {
            player.a-=0.04f;
        }
        else if (keydown[K_RIGHT])
        {
            player.a+=0.04f;
        }
	
		if (keydown[K_UP]&&keydown[K_DOWN])
		{
		}
        else if (keydown[K_UP])
        {
			player.x+=cos(player.a)*0.3f;
			player.y+=sin(player.a)*0.3f;
        }
        else if (keydown[K_DOWN])
        {
			player.x-=cos(player.a)*0.3f;
			player.y-=sin(player.a)*0.3f;
        }
	
		if (keydown[K_SPACE])
		{
			if (player.x>102)
			{
				player.x-=0.3f;
			}
			if (player.x<98)
			{
				player.x+=0.3f;
			}
			
			if (player.y>102)
			{
				player.y-=0.3f;
			}
			if (player.y<98)
			{
				player.y+=0.3f;
			}
			
		}
	
	updateSounds();
}     

void DrawGame()
{    
	
	
	
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	
	glOrtho(1.0f, 320, 240, 1.0f, -1.0f, 1.0f);
	
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	
	glColor4f(1, 1, 1,1);
	
	
	glBegin(GL_LINES);
	glVertex2f(player.x, player.y);
	glVertex2f(player.x+cos(player.a)*LINE_LENGTH, player.y+sin(player.a)*LINE_LENGTH);
	glEnd();
	
	glBegin(GL_TRIANGLES);
	
	for (int i=0;i<emit.size();i++)
	{
		glVertex2f(emit[i].x-1,emit[i].y-1);
		glVertex2f(emit[i].x-1,emit[i].y+1);
		glVertex2f(emit[i].x+1,emit[i].y-1);
		glVertex2f(emit[i].x-1,emit[i].y+1);
		glVertex2f(emit[i].x+1,emit[i].y-1);
		glVertex2f(emit[i].x+1,emit[i].y+1);
	}
	glEnd();
	
	SDL_GL_SwapBuffers();        
}

int main(int argc, char *argv[])
{
	//SDL_Surface *screen;
	Uint32 videoflags = SDL_OPENGL;
    videoflags |= SDL_FULLSCREEN;
	int    done;
        SDL_Event event;
    
    if ( SDL_Init(SDL_INIT_VIDEO) != 0 ) 
    {
        printf("Unable to initialize SDL: %s\n", SDL_GetError());
        exit(1);
    }
     
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 ); // *new*
    SDL_GL_SetAttribute( SDL_GL_SWAP_CONTROL, 1 );
     
    SDL_Surface* screen = SDL_SetVideoMode( 640, 480, 32, videoflags ); // *changed* 
    
    glEnable( GL_TEXTURE_2D );
     
    glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
     
    glViewport( 0, 0, 640, 480 );
        
        
    SDL_WM_SetCaption( "Hello World", NULL );
            
        
    Uint32 rmask, gmask, bmask, amask;
    #if SDL_BYTEORDER == SDL_BIG_ENDIAN
    rmask = 0xff000000;
    gmask = 0x00ff0000;
    bmask = 0x0000ff00;
    amask = 0x000000ff;
    #else
    rmask = 0x000000ff;
    gmask = 0x0000ff00;
    bmask = 0x00ff0000;
    amask = 0xff000000;
    #endif

    SDL_ShowCursor(SDL_DISABLE);
        
    init_sound();
        
    InitGame();
    
	done = 0;
    while( !done ) { 
        //Start the frame timer 
        fps.start(); 
		/* Check for events */
		while ( SDL_PollEvent(&event) ) {
			switch (event.type) {

				case SDL_MOUSEMOTION:
					break;
				case SDL_MOUSEBUTTONDOWN:
					break;
				case SDL_KEYDOWN:
                    switch (event.key.keysym.sym)
                    {
                        case SDLK_ESCAPE:
                            done=1;
                            break;
                        case SDLK_LEFT:
                            keydown[K_LEFT]=true;
                            break;
                        case SDLK_RIGHT:
                            keydown[K_RIGHT]=true;
                            break;
                        case SDLK_UP:
                            keydown[K_UP]=true;
                            break;
                        case SDLK_DOWN:
                            keydown[K_DOWN]=true;
                            break;
                        case SDLK_SPACE:
                            keydown[K_SPACE]=true;
                            break;
                    }
                    
					/* Any keypress quits the app... */
                    break;
                case SDL_KEYUP:                
                    switch (event.key.keysym.sym)
                    {
                        case SDLK_ESCAPE:
                            done=1;
                            break;
                        case SDLK_LEFT:
                            keydown[K_LEFT]=false;
                            break;
                        case SDLK_RIGHT:
                            keydown[K_RIGHT]=false;
                            break;
                        case SDLK_UP:
                            keydown[K_UP]=false;
                            break;
                        case SDLK_DOWN:
                            keydown[K_DOWN]=false;
                            break;
                        case SDLK_SPACE:
                            keydown[K_SPACE]=false;
                            break;
                    }
                    break;
				case SDL_QUIT:
					done = 1;
					break;
				default:
					break;
			}
		}
        
        GameLogic();
        
		glClear( GL_COLOR_BUFFER_BIT );
		
//        DrawGame();
      
        
        if( SDL_Flip( screen ) == -1 ) 
        { 
            return 1; 
        } 
        frame++;
        
        if( fps.get_ticks() < 1000 / FRAMES_PER_SECOND  )
        {
            SDL_Delay( ( 1000 / FRAMES_PER_SECOND ) - fps.get_ticks() );        
        }
	}
	
	/* Clean up the SDL library */
	SDL_Quit();
	return(0);
}
